<?php

require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-action.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'public/rest/class-public-rest-action.php';

/**
 * Returns the open deficiencies
 *
 * An action class.
 *
 * @link       http://www.servicetrade.com
 * @since      2.0.0
 * @package    Servicetrade_Service_Portal
 * @subpackage Servicetrade_Service_Portal/public/rest/actions
 * @author     ServiceTrade <support@servicetrade.com>
 * @example    http://localhost:8888/wp-json/servicetrade-service-portal/v2/public/deficiencies/open
 */
class Servicetrade_Service_Portal_Public_Actions_Rest_Deficiencies_Open_Get_Action extends Servicetrade_Service_Portal_Public_Rest_Action {

  private $filterLocationId;
  private $filterServiceLineId;

    public function __construct($filterLocationId,
                                                            $filterServiceLineId) {
    $this->filterLocationId = $filterLocationId;
        $this->filterServiceLineId = $filterServiceLineId;
  }


  function run() {
    $user = $this->get_authenticated_user();
    if(null === $user) {
            return $this->create_rest_error_not_authorized();
    }

    $contactId = $user->getContactId();

    $query = array();
    if(!empty($this->filterLocationId)) {
      $query['locationId'] = $this->filterLocationId;
    } else {
      /* The API requires a location Id. If none exists we
      return an empty result set
      */
      return new WP_REST_Response( array("deficiencies" => []), 200 );
    }

    /*
    ** Can't be filtered per API docs
    if(!empty($this->filterServiceLineId)) {
      $query[] = array('serviceLineId' => $this->filterServiceLineId);
    }
    */

    $location = $this->st_api()->get( '/location/' . $this->filterLocationId);
    if(empty($location) || !isset( $location['id'] )) {
      /**
       * The location was not found - we treat this as
       * a soft error and return an empty result.
       */
      return new WP_REST_Response( array("deficiencies" => []), 200 );
    }

     $apiResult = $this->api_get($this->build_location_deficiencies_query());

    if(!isset( $apiResult['deficiencies'] )) {
      return $this->create_rest_error_failed_request();
    }
     $mapParams = [
         'asset' => 'assets',
         'serviceLine' => 'serviceLines',
         'serviceRequests' => 'serviceRequests',
         'quote' => 'quotes',
     ];
     $deficiencies =  Servicetrade_Helper_Sideloadmapper::map_sideloaded_data( $apiResult, 'deficiencies', $mapParams );

    $mappedResults = array();

    foreach ($deficiencies as $result) {

      $mappedServiceLines = array();

      if(!empty($result['serviceLine'] )) {
        $mappedServiceLines[] = array(
          'id' => $result['serviceLine']['id'],
          'name' => $result['serviceLine']['name'],
        );
      }

        // Filter fixed and invalid deficiencies
        if (in_array($result['status'], array('fixed', 'invalid'))) {
            continue;
        }

      // Filter service lines
      if(!empty($this->filterServiceLineId)) {
        $foundServiceLine = false;

        foreach($mappedServiceLines as $serviceLine) {
          if($this->filterServiceLineId == $serviceLine['id']) {
            $foundServiceLine = true;
          }
        }

        if(!$foundServiceLine) {
          continue;
        }
        }

         $quotes = [];
         $lastQuote = $quoteAmount = $encryptedQuoteLinkData = null;
         if ($result['serviceRequests']) {
             $quotes = array_map(function($s) { return $s['quote']; }, $result['serviceRequests']);
             $quotes = array_values(array_filter($quotes, function($q) {
                 return in_array($q['status'], ['submitted', 'accepted']);
             }));
             $lastQuote = $quotes ? $quotes[0] : [];
             $quoteAmount = array_sum(array_map(function($q) { return $q['totalPrice']; }, $quotes));
         }
         $statusColor = '';
         $status = ucwords(str_replace('_', ' ', $result['status']));
         if ($status == 'Fixed') {
             $statusColor = 'green';
         } else if ($status != 'Fixed' && $lastQuote) {
             $statusColor = 'blue';
             $status = 'Quoted';
            $quoteLinkData = ['quoteId' => $lastQuote['id'], 'contactId' => $contactId];
            $encryptedQuoteLinkData = Servicetrade_Service_Portal_Public_Authenticated_User::encryptUserData($quoteLinkData);
         } else if (in_array($status, ['New','Verified'])) {
             $status = 'Open';
         }

      $mappedResults[] = array(
        'id' => $result['id'], // The deficiency number
        'locationId' => $this->filterLocationId,
        'locationName' => $location['name'],
        'reportedOn' => $result['reportedOn'],
        'assetId' => $result['asset']['id'],
        'assetName' => $result['asset']['name'],
        'serviceLines' => $mappedServiceLines,
        'description' => $result['description'],
        'severity' => $result['severity'],
        'resolution' => $result['resolution'],
        'status' => $status,
        'statusColor' => $statusColor,
        'quoteId' => $lastQuote ? $lastQuote['id'] : null,
        'quoteAmount' => $quoteAmount,
        'quoteLinkData' => $encryptedQuoteLinkData,
      );
    }

    $restResponse =  array(
      "deficiencies" => $mappedResults
    );
    return new WP_REST_Response( $restResponse, 200 );
  }

    protected function build_location_deficiencies_query() {
        $query = array(
            'locationId' => $this->filterLocationId,
            '_sideload' => 'deficiency.serviceRequests,deficiency.asset,deficiency.serviceLine,serviceRequest.quote'
        );
    return '/deficiency?' . http_build_query($query);
    }
}
